@@ -18,7 +18,7 @@ server = function(input, output, session) {
 
   myPars = reactiveValues(
     print = FALSE,          # if TRUE, some functions print a message to the console when called
-    debugQn = FALSE,             # for debugging - click "?" to step into the code
+    debugQn = FALSE,        # for debugging - click "?" to step into the code
     zoomFactor = 2,         # zoom buttons change time zoom by this factor
     zoomFactor_freq = 1.5,  # same for frequency
     shinyTip_show = 1000,      # delay until showing a tip (ms)
@@ -59,6 +59,7 @@ server = function(input, output, session) {
   })
   observeEvent(input$append, {
     myPars$out = try(read.csv('www/temp.csv', stringsAsFactors = FALSE))
+    myPars$out = unique(myPars$out)  # remove duplicate rows
     removeModal()
   })
 
@@ -213,6 +214,14 @@ server = function(input, output, session) {
     #                   value = c(0, min(def_form['spectrum_xlim', 'default'], myPars$nyquist)),
     #                   max = myPars$nyquist)
     myPars$dur = length(myPars$temp_audio@left) * 1000 / myPars$temp_audio@samp.rate
+    myPars$myAudio_list = list(
+      sound = myPars$myAudio,
+      samplingRate = myPars$samplingRate,
+      scale = myPars$maxAmpl,
+      timeShift = 0,
+      ls = length(myPars$myAudio),
+      duration = myPars$dur / 1000
+    )
     myPars$time = seq(1, myPars$dur, length.out = myPars$ls)
     myPars$spec_xlim = c(0, min(myPars$initDur, myPars$dur))
     if (!is.finite(myPars$spec_xlim[2])) browser()  # weird glitches
@@ -252,9 +261,8 @@ server = function(input, output, session) {
     # matrix and re-draw manually with soundgen:::filled.contour.mod
     if (!is.null(myPars$myAudio)) {  # & is.null(myPars$spec)
       if (myPars$print) print('Extracting spectrogram...')
-      temp_spec = try(spectrogram(
-        myPars$myAudio,
-        samplingRate = myPars$samplingRate,
+      temp_spec = try(soundgen:::.spectrogram(
+        myPars$myAudio_list,
         dynamicRange = input$dynamicRange,
         windowLength = input$windowLength,
         step = input$step,
@@ -262,13 +270,16 @@ server = function(input, output, session) {
         zp = 2 ^ input$zp,
         contrast = input$specContrast,
         brightness = input$specBrightness,
-        output = 'processed',
+        blur = c(input$blur_freq, input$blur_time),
+        specType = input$specType,
+        output = 'all',
         plot = FALSE
       ))
       if (!inherits(temp_spec, 'try-error') &&
           length(temp_spec) > 0 &&
-          is.matrix(temp_spec))
-        myPars$spec = temp_spec
+          is.matrix(temp_spec$processed))
+        myPars$spec = temp_spec$processed
+      myPars$reassigned = temp_spec$reassigned
     }
   })
 
@@ -397,29 +408,41 @@ server = function(input, output, session) {
         text(x = 5, y = 5,
              labels = 'Upload wav/mp3 file(s) to begin...\nSuggested max duration ~30 s')
       } else {
-        if (input$spec_colorTheme == 'bw') {
-          color.palette = function(x) gray(seq(from = 1, to = 0, length = x))
-        } else if (input$spec_colorTheme == 'seewave') {
-          color.palette = seewave::spectro.colors
+        if (input$specType != 'reassigned') {
+          # rasterized spectrogram
+          soundgen:::filled.contour.mod(
+            x = as.numeric(colnames(myPars$spec_trimmed)),
+            y = as.numeric(rownames(myPars$spec_trimmed)),
+            z = t(myPars$spec_trimmed),
+            levels = seq(0, 1, length = 30),
+            color.palette = soundgen:::switchColorTheme(input$spec_colorTheme),
+            log = if (input$spec_yScale == 'log') 'y' else '',
+            yScale = if (input$spec_yScale %in% c('bark', 'mel', 'ERB')) input$spec_yScale else 'orig',
+            xlim = myPars$spec_xlim,
+            xaxt = 'n',
+            xaxs = 'i', xlab = '',
+            ylab = '',
+            main = '',
+            ylim = input$spec_ylim
+          )
         } else {
-          colFun = match.fun(input$spec_colorTheme)
-          color.palette = function(x) rev(colFun(x))
+          # unrasterized reassigned spectrogram
+          soundgen:::plotUnrasterized(
+            myPars$reassigned,
+            levels = seq(0, 1, length = 30),
+            color.palette = soundgen:::switchColorTheme(input$spec_colorTheme),
+            log = if (input$spec_yScale == 'log') 'y' else '',
+            yScale = if (input$spec_yScale %in% c('bark', 'mel', 'ERB'))
+              input$spec_yScale else 'orig',
+            xlim = myPars$spec_xlim,
+            xaxt = 'n',
+            xaxs = 'i', xlab = '',
+            ylab = '',
+            main = '',
+            ylim = input$spec_ylim,
+            cex = input$reass_cex
+          )
         }
-        soundgen:::filled.contour.mod(
-          x = as.numeric(colnames(myPars$spec_trimmed)),
-          y = as.numeric(rownames(myPars$spec_trimmed)),
-          z = t(myPars$spec_trimmed),
-          levels = seq(0, 1, length = 30),
-          color.palette = color.palette,
-          log = if (input$spec_yScale == 'log') 'y' else '',
-          yScale = if (input$spec_yScale %in% c('bark', 'mel', 'ERB')) input$spec_yScale else 'orig',
-          xlim = myPars$spec_xlim,
-          xaxt = 'n',
-          xaxs = 'i', xlab = '',
-          ylab = '',
-          main = '',
-          ylim = input$spec_ylim
-        )
 
         # Add text label of file name
         if (input$spec_yScale == 'bark') {
@@ -621,6 +644,7 @@ server = function(input, output, session) {
 
   new_annotation = function() {
     if (myPars$print) print('Creating a new annotation...')
+    Sys.sleep(0.5)  # wait for 500 ms, otherwise label not saved if OK is clicked too fast (why?)
     new = data.frame(
       # idx = ifelse(is.null(myPars$ann), 1, nrow(myPars$ann) + 1),
       file = myPars$myAudio_filename,
@@ -1049,12 +1073,15 @@ server = function(input, output, session) {
   shinyBS::addTooltip(session, id='spec_ylim', title = "Range of displayed frequencies, kHz", placement="right", trigger="hover", options = list(delay = list(show = 1000, hide = 0)))
   shinyBS::addTooltip(session, id='windowLength', title = 'Length of STFT window, ms.', placement="right", trigger="hover", options = list(delay = list(show = 1000, hide = 0)))
   shinyBS::addTooltip(session, id='step', title = 'Step between analysis frames, ms', placement="right", trigger="hover", options = list(delay = list(show = 1000, hide = 0)))
+  shinyBS::addTooltip(session, id='specType', title = 'Spectrogram type, argument "specType" in spectrogram()', placement="right", trigger="hover", options = list(delay = list(show = 1000, hide = 0)))
   # shinyBS::addTooltip(session, id='overlap', title = 'Overlap between analysis frames, %', placement="right", trigger="hover", options = list(delay = list(show = 1000, hide = 0)))
   shinyBS::addTooltip(session, id='dynamicRange', title = 'Dynamic range, dB', placement="right", trigger="hover", options = list(delay = list(show = 1000, hide = 0)))
   shinyBS::addTooltip(session, id='spec_cex', title = "Magnification coefficient controlling the size of points showing pitch candidates", placement="right", trigger="hover", options = list(delay = list(show = 1000, hide = 0)))
   shinyBS::addTooltip(session, id='spec_yScale', title = 'Frequency scale', placement="below", trigger="hover", options = list(delay = list(show = 1000, hide = 0)))
   shinyBS::addTooltip(session, id='specContrast', title = 'Regulates the contrast of the spectrogram', placement="below", trigger="hover", options = list(delay = list(show = 1000, hide = 0)))
   shinyBS::addTooltip(session, id='specBrightness', title = 'Regulates the brightness of the spectrogram', placement="below", trigger="hover", options = list(delay = list(show = 1000, hide = 0)))
+  shinyBS::addTooltip(session, id='blur_freq', title = 'Gaussian filter of frequency: >0 = blur, <0 = unblur (sharpen)', placement="below", trigger="hover", options = list(delay = list(show = 1000, hide = 0)))
+  shinyBS::addTooltip(session, id='blur_time', title = 'Gaussian filter of time: >0 = blur, <0 = unblur (sharpen)', placement="below", trigger="hover", options = list(delay = list(show = 1000, hide = 0)))
   shinyBS::addTooltip(session, id='zp', title = 'Zero padding: 8 means 2^8 = 256, etc.', placement="right", trigger="hover", options = list(delay = list(show = 1000, hide = 0)))
   shinyBS::addTooltip(session, id='wn', title = 'Type of STFT window', placement="right", trigger="hover", options = list(delay = list(show = 1000, hide = 0)))
   shinyBS::addTooltip(session, id='spec_maxPoints', title = 'The number of points to plot in the spectrogram (smaller = faster, but low resolution)', placement="below", trigger="hover", options = list(delay = list(show = 1000, hide = 0)))
