@@ -1,6 +1,6 @@
 # formant_app()
 #
-# To do: hotkey A to create new annotation (see annotation_app); maybe an option to have log-spectrogram and log-spectrum (a bit tricky b/c all layers have to be adjusted); maybe remove the buggy feature of editing formant freq in the button as text, just display current value there (but then how to make it NA?); LPC saves all avail formants - check beh when changing nFormants across annotations & files; from-to in play sometimes weird (stops audio while cursor is still moving); highlight smts disappears in ann_table (buggy! tricky!); load audio upon session start; maybe arbitrary number of annotation tiers
+# To do: deleting a formant in the button also deletes the annotation (bug); maybe an option to have log-spectrogram and log-spectrum (a bit tricky b/c all layers have to be adjusted); maybe remove the buggy feature of editing formant freq in the button as text, just display current value there (but then how to make it NA?); LPC saves all avail formants - check beh when changing nFormants across annotations & files; from-to in play sometimes weird (stops audio while cursor is still moving); highlight smts disappears in ann_table (buggy! tricky!); load audio upon session start; maybe arbitrary number of annotation tiers
 
 # Start with a fresh R session and run the command options(shiny.reactlog=TRUE)
 # Then run your app in a show case mode: runApp('inst/shiny/formant_app', display.mode = "showcase")
@@ -68,6 +68,7 @@ server = function(input, output, session) {
   })
   observeEvent(input$append, {
     myPars$out = try(read.csv('www/temp.csv', stringsAsFactors = FALSE))
+    myPars$out = unique(myPars$out)  # remove duplicate rows
     removeModal()
   })
 
@@ -134,6 +135,10 @@ server = function(input, output, session) {
           !any(!oblig_cols %in% colnames(user_ann))) {
         idx_missing = which(apply(user_ann[, oblig_cols], 1, function(x) any(is.na(x))))
         if (length(idx_missing) > 0) user_ann = user_ann[-idx_missing, ]
+        need_cols = c('label', 'dF', 'vtl', myPars$ff)
+        missing_cols = need_cols[which(!need_cols %in% colnames(user_ann))]
+        if (length(missing_cols) > 0) user_ann[, missing_cols] = NA
+        user_ann = user_ann[, c(oblig_cols, need_cols)]
         if (nrow(user_ann) > 0) {
           if (is.null(myPars$out)) {
             myPars$out = user_ann
@@ -231,6 +236,14 @@ server = function(input, output, session) {
     #                   value = c(0, min(def_form['spectrum_xlim', 'default'], myPars$nyquist)),
     #                   max = myPars$nyquist)
     myPars$dur = length(myPars$temp_audio@left) * 1000 / myPars$temp_audio@samp.rate
+    myPars$myAudio_list = list(
+      sound = myPars$myAudio,
+      samplingRate = myPars$samplingRate,
+      scale = myPars$maxAmpl,
+      timeShift = 0,
+      ls = length(myPars$myAudio),
+      duration = myPars$dur / 1000
+    )
     myPars$time = seq(1, myPars$dur, length.out = myPars$ls)
     myPars$spec_xlim = c(0, min(myPars$initDur, myPars$dur))
     if (!is.finite(myPars$spec_xlim[2])) browser()  # weird glitches
@@ -277,8 +290,8 @@ server = function(input, output, session) {
     # matrix and re-draw manually with soundgen:::filled.contour.mod
     if (!is.null(myPars$myAudio)) {  # & is.null(myPars$spec)
       if (myPars$print) print('Extracting spectrogram...')
-      temp_spec = try(spectrogram(
-        myPars$myAudio,
+      temp_spec = try(soundgen:::.spectrogram(
+        myPars$myAudio_list,
         samplingRate = myPars$samplingRate,
         dynamicRange = input$dynamicRange,
         windowLength = input$windowLength,
@@ -287,13 +300,16 @@ server = function(input, output, session) {
         zp = 2 ^ input$zp,
         contrast = input$specContrast,
         brightness = input$specBrightness,
-        output = 'processed',
+        blur = c(input$blur_freq, input$blur_time),
+        specType = input$specType,
+        output = 'all',
         plot = FALSE
       ))
       if (!inherits(temp_spec, 'try-error') &&
           length(temp_spec) > 0 &&
-          is.matrix(temp_spec))
-        myPars$spec = temp_spec
+          is.matrix(temp_spec$processed))
+        myPars$spec = temp_spec$processed
+      myPars$reassigned = temp_spec$reassigned
     }
   })
 
@@ -348,10 +364,16 @@ server = function(input, output, session) {
         label = HTML("<img src='icons/plus.png' width = '25px'>"), # '+',
         class = "buttonInline"
       )),
+      list(actionButton(
+        inputId = 'defaultFmtBtn',
+        label = HTML("<img src='icons/update.png' width = '30px'>"),
+        class = "buttonInline"
+      )),
       list(
-        shinyBS:::bsPopover(id = 'fDiv_1', title = NULL, content = 'Click to select, type a number to correct or NA to treat as missing, empty to reset to default', placement = "right", trigger = "hover", options = list(delay = list(show = 1000, hide = 0))),
+        shinyBS:::bsPopover(id = 'fDiv_1', title = NULL, content = 'Click to select, type a number to correct, any text or NA to treat as missing, empty to reset to default', placement = "right", trigger = "hover", options = list(delay = list(show = 1000, hide = 0))),
         shinyBS::bsPopover(id='remF', title = NULL, content = 'Remove one formant', placement = "right", trigger = "hover", options = list(delay = list(show = 1000, hide = 0))),
-        shinyBS::bsPopover(id='addF', title = NULL, content = 'Add one formant', placement = "right", trigger = "hover", options = list(delay = list(show = 1000, hide = 0)))
+        shinyBS::bsPopover(id='addF', title = NULL, content = 'Add one formant', placement = "right", trigger = "hover", options = list(delay = list(show = 1000, hide = 0))),
+        shinyBS::bsPopover(id='defaultFmtBtn', title = NULL, content = 'Reset formant values to defaults as measured', placement = "right", trigger = "hover", options = list(delay = list(show = 1000, hide = 0)))
       )
     )
   })
@@ -573,27 +595,40 @@ server = function(input, output, session) {
         text(x = 5, y = 5,
              labels = 'Upload wav/mp3 file(s) to begin...\nSuggested max duration ~30 s')
       } else {
-        if (input$spec_colorTheme == 'bw') {
-          color.palette = function(x) gray(seq(from = 1, to = 0, length = x))
-        } else if (input$spec_colorTheme == 'seewave') {
-          color.palette = seewave::spectro.colors
+        if (input$specType != 'reassigned') {
+          # rasterized spectrogram
+          soundgen:::filled.contour.mod(
+            x = as.numeric(colnames(myPars$spec_trimmed)),
+            y = as.numeric(rownames(myPars$spec_trimmed)),
+            z = t(myPars$spec_trimmed),
+            levels = seq(0, 1, length = 30),
+            color.palette = soundgen:::switchColorTheme(input$spec_colorTheme),
+            log = '',
+            yScale = 'linear',
+            xlim = myPars$spec_xlim,
+            xaxt = 'n',
+            xaxs = 'i', xlab = '',
+            ylab = '',
+            main = '',
+            ylim = input$spec_ylim
+          )
         } else {
-          colFun = match.fun(input$spec_colorTheme)
-          color.palette = function(x) rev(colFun(x))
+          # unrasterized reassigned spectrogram
+          soundgen:::plotUnrasterized(
+            myPars$reassigned,
+            levels = seq(0, 1, length = 30),
+            color.palette = soundgen:::switchColorTheme(input$spec_colorTheme),
+            log = '',
+            yScale = 'linear',
+            xlim = myPars$spec_xlim,
+            xaxt = 'n',
+            xaxs = 'i', xlab = '',
+            ylab = '',
+            main = '',
+            ylim = input$spec_ylim,
+            cex = input$reass_cex
+          )
         }
-        soundgen:::filled.contour.mod(
-          x = as.numeric(colnames(myPars$spec_trimmed)),
-          y = as.numeric(rownames(myPars$spec_trimmed)),
-          z = t(myPars$spec_trimmed),
-          levels = seq(0, 1, length = 30),
-          color.palette = color.palette,
-          xlim = myPars$spec_xlim,
-          xaxt = 'n',
-          xaxs = 'i', xlab = '',
-          ylab = 'Frequency, kHz',
-          main = '',
-          ylim = input$spec_ylim
-        )
 
         # Add text label of file name
         ran_x = myPars$spec_xlim[2] - myPars$spec_xlim[1]
@@ -654,7 +689,7 @@ server = function(input, output, session) {
 
       # Add formant tracks
       if (!is.null(myPars$formantTracks)) {
-        for (f in 2:(input$nFormants + 1)) {
+        for (f in 2:(length(myPars$ff) + 1)) {
           points(myPars$formantTracks$time,
                  myPars$formantTracks[, f] / 1000,
                  pch = 16,
@@ -1259,12 +1294,7 @@ server = function(input, output, session) {
       sel_anal = max(1, round(myPars$regionToAnalyze[1] / 1000 * myPars$samplingRate)) :
         min(myPars$ls, round(myPars$regionToAnalyze[2] / 1000 * myPars$samplingRate))
       temp_anal = try(soundgen:::.analyze(
-        list(sound = myPars$myAudio[sel_anal],
-             samplingRate = myPars$samplingRate,
-             scale = myPars$maxAmpl,
-             timeShift = myPars$regionToAnalyze[1] / 1000,
-             ls = length(sel_anal),
-             duration = length(sel_anal) / myPars$samplingRate),
+        myPars$myAudio_list,
         windowLength = input$windowLength_lpc,
         step = input$step_lpc,
         wn = input$wn_lpc,
@@ -1291,6 +1321,7 @@ server = function(input, output, session) {
         myPars$temp_anal = temp_anal
         myPars$nMeasuredFmts = length(grep('_freq', colnames(myPars$temp_anal)))
         myPars$allF_colnames = paste0('f', 1:myPars$nMeasuredFmts, '_freq')
+        myPars$ff = myPars$ff[1:min(input$nFormants, myPars$nMeasuredFmts)]
         myPars$temp_anal = myPars$temp_anal[, c('time', myPars$allF_colnames)]
         colnames(myPars$temp_anal) = c('time', paste0('F', 1:myPars$nMeasuredFmts))
         for (c in colnames(myPars$temp_anal)) {
@@ -1339,7 +1370,7 @@ server = function(input, output, session) {
             myPars$formantTracks$time <= myPars$ann$to[myPars$currentAnn]
         )
       })
-      fMat = myPars$formantTracks[idx, 2:(input$nFormants + 1)]
+      fMat = myPars$formantTracks[idx, 2:(length(myPars$ff) + 1)]
       myPars$formants = apply(fMat, 2, function(x)
         round(do.call(input$summaryFun, list(x, na.rm = TRUE))))
       # myPars$bandwidth ?
@@ -1358,6 +1389,18 @@ server = function(input, output, session) {
   }
   observeEvent(myPars$formantTracks, avFmPerSel())
 
+  observeEvent(input$defaultFmtBtn, {
+    # browser()
+    for (f in 1:input$nFormants) {
+      fn = paste0('F', f, '_text')
+      updateTextInput(
+        session, fn,
+        value = as.character(myPars$formants[f]))
+    }
+    myPars$ann[myPars$currentAnn, myPars$ff] = myPars$formants[myPars$ff]
+    updateVTL()
+  })
+
   observeEvent(myPars$ann, {
     if (myPars$print) print('Drawing ann_table...')
     if (!is.null(myPars$ann)) {
@@ -1789,20 +1832,25 @@ server = function(input, output, session) {
                              input$pitch / 1.11, input$pitch / 1.5))
       if (input$adaptivePitch)
         pitch$value = pitch$value * 17 / myPars$ann$vtl[myPars$currentAnn]
-      temp_s = soundgen(
+      ff_vector = as.numeric(myPars$ann[myPars$currentAnn, myPars$ff])
+      # drop NAs in formant frequencies, otherwise soundgen can't synthesize them
+      ff_vector = ff_vector[!is.na(ff_vector)]
+      temp_s = try(soundgen(
         sylLen = 300 * myPars$samplingRate_idx,
         pitch = pitch,
-        formants = as.numeric(myPars$ann[myPars$currentAnn, myPars$ff]),
-        temperature = .001, tempEffects = list(formDisp = 0, formDrift = 0))
-      if (input$audioMethod == 'Browser') {
-        # save a temporary file and play with the browser
-        seewave::savewav(temp_s,
-                         f = 16000 * myPars$samplingRate_idx,
-                         filename = 'www/temp.wav')
-        shinyjs::js$play_file(filename = 'temp.wav')
-      } else {
-        # play directly in R without saving to disk
-        playme(temp_s, samplingRate = 16000 * myPars$samplingRate_idx)
+        formants = ff_vector,
+        temperature = .001, tempEffects = list(formDisp = 0, formDrift = 0)))
+      if (!inherits(temp_s, 'try-error')) {
+        if (input$audioMethod == 'Browser') {
+          # save a temporary file and play with the browser
+          seewave::savewav(temp_s,
+                           f = 16000 * myPars$samplingRate_idx,
+                           filename = 'www/temp.wav')
+          shinyjs::js$play_file(filename = 'temp.wav')
+        } else {
+          # play directly in R without saving to disk
+          playme(temp_s, samplingRate = 16000 * myPars$samplingRate_idx)
+        }
       }
     }
   }
@@ -1842,11 +1890,14 @@ server = function(input, output, session) {
   shinyBS::addTooltip(session, id='spec_ylim', title = "Range of displayed frequencies, kHz", placement="right", trigger="hover", options = list(delay = list(show = 1000, hide = 0)))
   shinyBS::addTooltip(session, id='windowLength', title = 'Length of STFT window for plotting the spectrogram', placement="rights", trigger="hover", options = list(delay = list(show = 1000, hide = 0)))
   shinyBS::addTooltip(session, id='step', title = 'Step between analysis frames, ms', placement="right", trigger="hover", options = list(delay = list(show = 1000, hide = 0)))
+  shinyBS::addTooltip(session, id='specType', title = 'Spectrogram type, argument "specType" in spectrogram(). Visualization only, does NOT affect formant tracking with LPC', placement="right", trigger="hover", options = list(delay = list(show = 1000, hide = 0)))
   # shinyBS::addTooltip(session, id='overlap', title = 'Overlap between analysis frames, %', placement="right", trigger="hover", options = list(delay = list(show = 1000, hide = 0)))
   shinyBS::addTooltip(session, id='dynamicRange', title = 'Dynamic range, dB', placement="right", trigger="hover", options = list(delay = list(show = 1000, hide = 0)))
-  shinyBS::addTooltip(session, id='spec_cex', title = "Magnification coefficient controlling the size of points showing pitch candidates", placement="right", trigger="hover", options = list(delay = list(show = 1000, hide = 0)))
+  shinyBS::addTooltip(session, id='spec_cex', title = "Magnification coefficient controlling the size of points showing formant candidates", placement="right", trigger="hover", options = list(delay = list(show = 1000, hide = 0)))
   shinyBS::addTooltip(session, id='specContrast', title = 'Regulates the contrast of the spectrogram', placement="below", trigger="hover", options = list(delay = list(show = 1000, hide = 0)))
   shinyBS::addTooltip(session, id='specBrightness', title = 'Regulates the brightness of the spectrogram', placement="below", trigger="hover", options = list(delay = list(show = 1000, hide = 0)))
+  shinyBS::addTooltip(session, id='blur_freq', title = 'Gaussian filter of frequency: >0 = blur, <0 = unblur (sharpen)', placement="below", trigger="hover", options = list(delay = list(show = 1000, hide = 0)))
+  shinyBS::addTooltip(session, id='blur_time', title = 'Gaussian filter of time: >0 = blur, <0 = unblur (sharpen)', placement="below", trigger="hover", options = list(delay = list(show = 1000, hide = 0)))
   shinyBS::addTooltip(session, id='zp', title = 'Zero padding: 8 means 2^8 = 256, etc.', placement="right", trigger="hover", options = list(delay = list(show = 1000, hide = 0)))
   shinyBS::addTooltip(session, id='wn', title = 'Type of STFT window', placement="right", trigger="hover", options = list(delay = list(show = 1000, hide = 0)))
   shinyBS::addTooltip(session, id='spec_maxPoints', title = 'The number of points to plot in the spectrogram (smaller = faster, but low resolution)', placement="below", trigger="hover", options = list(delay = list(show = 1000, hide = 0)))
